#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Raw Request

File : rf_raw_request.py

Brief : This script performs a raw request specified by the user
"""

import argparse
import json
import os
import redfish
import redfish_utilities
import sys
from redfish.messages import RedfishPasswordChangeRequiredError


def ifmatch_header(redfish_obj, path, headers=None):
    """
    Generates If-Match header for PATCH and PUT operations

    Args:
        redfish_obj: The Redfish client object with an open session
        path: The URI of the resource
        headers: Dictionary of HTTP headers to provide in the request

    Returns:
        Updated dictionary of HTTP headers with If-Match, if an ETag was found
    """

    if headers is None:
        headers = {}
    try:
        response = redfish_obj.get(path)
        etag = response.getheader("ETag")
        if etag is not None:
            headers["If-Match"] = etag
    except Exception:
        pass
    return headers


# Get the input arguments
argget = argparse.ArgumentParser(description="A tool perform a raw request to a Redfish service")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument(
    "--method",
    "-m",
    type=str,
    required=False,
    help="The HTTP method to perform; performs GET if not specified",
    default="GET",
    choices=["GET", "HEAD", "POST", "PATCH", "PUT", "DELETE"],
)
argget.add_argument("--request", "-req", type=str, required=True, help="The URI for the request")
argget.add_argument(
    "--body",
    "-b",
    type=str,
    required=False,
    help="The body to provide with the request; can be a JSON string for a JSON request, a filename to send binary data, or an unstructured string",
)
argget.add_argument(
    "--verbose",
    "-v",
    action="store_true",
    help="Indicates if HTTP response codes and headers are displayed",
    default=False,
)
args = argget.parse_args()

# Connect to the service
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

# Encode the body
# If the body argument points to a file, load the file
if args.body is not None and os.path.isfile(args.body):
    with open(args.body, mode="rb") as file:
        body = file.read()
else:
    # Not a file; either JSON or a raw string
    try:
        body = json.loads(args.body)
    except Exception:
        body = args.body
if body is None:
    # Default case if nothing resolves (empty JSON object)
    body = {}

headers = {}
# Perform the requested operation
if args.method == "HEAD":
    resp = redfish_obj.head(args.request)
elif args.method == "POST":
    resp = redfish_obj.post(args.request, body=body)
elif args.method == "PATCH":
    headers = ifmatch_header(redfish_obj, args.request, headers=headers)
    resp = redfish_obj.patch(args.request, body=body, headers=headers)
elif args.method == "PUT":
    headers = ifmatch_header(redfish_obj, args.request, headers=headers)
    resp = redfish_obj.put(args.request, body=body, headers=headers)
elif args.method == "DELETE":
    resp = redfish_obj.delete(args.request)
else:
    resp = redfish_obj.get(args.request)

# Print HTTP status and headers
if args.verbose:
    print("HTTP {}".format(resp.status))
    for header in resp.getheaders():
        print("{}: {}".format(header[0], header[1]))
    print()

# Print the response
if resp.status != 204:
    try:
        print(json.dumps(resp.dict, sort_keys=True, indent=4, separators=(",", ": ")))
    except Exception:
        # The response is either malformed JSON or not JSON at all
        print(resp.text)
else:
    print("No response body")

# Log out
redfish_utilities.logout(redfish_obj)
sys.exit(0)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Diagnostic Data

File : rf_diagnostic_data.py

Brief : This script uses the redfish_utilities module to collect diagnostic data from a log service
"""

import argparse
import datetime
import logging
import os
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(
    description="A tool to collect diagnostic data from a log service on a Redfish service"
)
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument(
    "--manager", "-m", type=str, nargs="?", default=False, help="The ID of the manager containing the log service"
)
argget.add_argument(
    "--system", "-s", type=str, nargs="?", default=False, help="The ID of the system containing the log service"
)
argget.add_argument(
    "--chassis", "-c", type=str, nargs="?", default=False, help="The ID of the chassis containing the log service"
)
argget.add_argument("--log", "-l", type=str, help="The ID of the log service")
argget.add_argument(
    "--type",
    "-type",
    type=redfish_utilities.diagnostic_data_types,
    help="The type of diagnostic data to collect; defaults to 'Manager' if not specified",
    choices=redfish_utilities.diagnostic_data_types,
    default=redfish_utilities.diagnostic_data_types.MANAGER,
)
argget.add_argument(
    "--oemtype",
    "-oemtype",
    type=str,
    help="The OEM-specific type of diagnostic data to collect; this option should only be used if the requested type is 'OEM'",
)
argget.add_argument(
    "--directory",
    "-d",
    type=str,
    help="The directory to save the diagnostic data; defaults to the current directory if not specified",
    default=".",
)
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
args = argget.parse_args()

# Determine the target log service based on the inputs
# Effectively if the user gives multiple targets, some will be ignored
container_type = redfish_utilities.log_container.MANAGER
container_id = None
if args.manager is not False:
    container_type = redfish_utilities.log_container.MANAGER
    container_id = args.manager
elif args.system is not False:
    container_type = redfish_utilities.log_container.SYSTEM
    container_id = args.system
elif args.chassis is not False:
    container_type = redfish_utilities.log_container.CHASSIS
    container_id = args.chassis

if args.debug:
    log_file = "rf_diagnostic_data-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_diagnostic_data Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    print("Collecting diagnostic data...")
    response = redfish_utilities.collect_diagnostic_data(
        redfish_obj, container_type, container_id, args.log, args.type, args.oemtype
    )
    response = redfish_utilities.poll_task_monitor(redfish_obj, response)
    filename, data = redfish_utilities.download_diagnostic_data(redfish_obj, response)

    # Save the file
    if not os.path.isdir(args.directory):
        os.makedirs(args.directory)
    path = os.path.join(args.directory, filename)
    name_parts = filename.split(".", 1)
    file_check = 0
    if len(name_parts) == 1:
        name_parts.append("")
    while os.path.isfile(path):
        # If the file already exists, build a new file name with a counter
        file_check = file_check + 1
        filename = "{}({}).{}".format(name_parts[0], file_check, name_parts[1])
        path = os.path.join(args.directory, filename)
    with open(path, "wb") as file:
        file.write(data)
    print("Saved diagnostic data to '{}'".format(path))
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Sensor List

File : rf_sensor_list.py

Brief : This script uses the redfish_utilities module to dump sensor info
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to walk a Redfish service and list sensor info")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument("--id", "-i", action="store_true", help="Construct sensor names using 'Id' values")
argget.add_argument("--name", "-n", action="store_true", help="Construct sensor names using 'Name' values")
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
args = argget.parse_args()

if args.debug:
    log_file = "rf_sensor_list-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_sensor_list Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    use_id = False
    if args.id:
        use_id = True
    if args.name:
        use_id = False
    # Get and print the sensor info
    sensors = redfish_utilities.get_sensors(redfish_obj, use_id)
    redfish_utilities.print_sensors(sensors)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Assembly Module

File : assembly.py

Brief : This file contains the definitions and functionalities for managing
        assemblies on a Redfish service
"""

from .messages import verify_response


class RedfishAssemblyNotFoundError(Exception):
    """
    Raised when an assembly index cannot be found
    """

    pass


class RedfishAssemblyNoBinaryDataError(Exception):
    """
    Raised when an assembly index does not contain binary data
    """

    pass


def get_assembly(context, uri):
    """
    Collects assembly information from a Redfish service

    Args:
        context: The Redfish client object with an open session
        uri: The URI of the assembly to get

    Returns:
        A list containing all assemblies from the URI
    """

    # Get the assembly
    assembly = context.get(uri)
    verify_response(assembly)
    return assembly.dict.get("Assemblies", [])


def print_assembly(assemblies, index=None):
    """
    Prints assembly information into a table

    Args:
        assemblies: An array of assembly information to print
        index: If specified, prints only the desired index
    """

    assembly_format_header = " {:5s} | {} {}"
    assembly_format = " {:5s} | {}: {}"
    assembly_properties = [
        "Model",
        "PartNumber",
        "SparePartNumber",
        "SKU",
        "SerialNumber",
        "Producer",
        "Vendor",
        "ProductionDate",
        "Version",
        "EngineeringChangeLevel",
    ]

    # If an index is specified, isolate to the one index
    if index is not None:
        if index < 0 or index >= len(assemblies):
            raise RedfishAssemblyNotFoundError(
                "Assembly contains {} entries; index {} is not valid".format(len(assemblies), index)
            )
        assemblies = [assemblies[index]]

    # Go through each assembly
    for assembly in assemblies:
        # Print the heading
        heading_details = []
        state = assembly.get("Status", {}).get("State")
        if state:
            heading_details.append(state)
        health = assembly.get("Status", {}).get("Health")
        if health:
            heading_details.append(health)
        heading_details = ", ".join(heading_details)
        if len(heading_details) != 0:
            heading_details = "(" + heading_details + ")"
        print(assembly_format_header.format(assembly["MemberId"], assembly["Name"], heading_details))

        # Print any of the found properties
        for property in assembly_properties:
            if property in assembly:
                print(assembly_format.format("", property, assembly[property]))


def download_assembly(context, assemblies, filepath, index=None):
    """
    Downloads the binary data of an assembly to a file

    Args:
        context: The Redfish client object with an open session
        assemblies: An array of assembly information
        filepath: The filepath to download the binary data
        index: The index into the assemblies array to download; if None, perform on index 0 if there's only 1 assembly
    """

    # Get the binary data URI
    binary_data_uri = get_assembly_binary_data_uri(assemblies, index)

    # Download the data and save it
    response = context.get(binary_data_uri)
    verify_response(response)
    with open(filepath, "wb") as binary_file:
        binary_file.write(response.read)


def upload_assembly(context, assemblies, filepath, index=None):
    """
    Uploads the binary data of a file to an assembly

    Args:
        context: The Redfish client object with an open session
        assemblies: An array of assembly information
        filepath: The filepath of the binary data to upload
        index: The index into the assemblies array to upload; if None, perform on index 0 if there's only 1 assembly
    """

    # Get the binary data URI
    binary_data_uri = get_assembly_binary_data_uri(assemblies, index)

    # Upload the binary data
    with open(filepath, "rb") as binary_file:
        data = binary_file.read()
    response = context.put(binary_data_uri, body=data)
    verify_response(response)


def get_assembly_binary_data_uri(assemblies, index=None):
    """
    Locates the binary data URI for a target assembly

    Args:
        assemblies: An array of assembly information
        index: The index into the assemblies array to download; if None, perform on index 0 if there's only 1 assembly

    Returns:
        A string containing the binary data URI
    """

    # If an index is specified, isolate to the one index
    if index is None:
        index = 0
        if len(assemblies) != 1:
            raise RedfishAssemblyNotFoundError(
                "Assembly contains {} entries; an index needs to be specified".format(len(assemblies))
            )
    else:
        if index < 0 or index >= len(assemblies):
            raise RedfishAssemblyNotFoundError(
                "Assembly contains {} entries; index {} is not valid".format(len(assemblies), index)
            )

    # Get the binary data URI
    binary_data_uri = assemblies[index].get("BinaryDataURI")
    if binary_data_uri is None:
        # No binary data
        raise RedfishAssemblyNoBinaryDataError("Assembly index {} does not contain binary data".format(index))
    return binary_data_uri



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Virtual Media

File : rf_virtual_media.py

Brief : This script uses the redfish_utilities module to manage virtual media
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to manage virtual media of a system")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument("--system", "-s", type=str, help="The ID of the system containing the virtual media")
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
subparsers = argget.add_subparsers(dest="command")
info_argget = subparsers.add_parser("info", help="Displays information about the virtual media for a system")
insert_argget = subparsers.add_parser("insert", help="Inserts virtual media for a system")
insert_argget.add_argument("--image", "-image", type=str, required=True, help="The URI of the image to insert")
insert_argget.add_argument("--id", "-i", type=str, help="The identifier of the virtual media instance to insert")
insert_argget.add_argument(
    "--notinserted",
    "-notinserted",
    action="store_false",
    help="Indicates if the media is to be marked as not inserted for the system",
    default=None,
)
insert_argget.add_argument(
    "--writable",
    "-writable",
    action="store_false",
    help="Indicates if the media is to be marked as writable for the system",
    default=None,
)
insert_argget.add_argument(
    "--mediatypes", "-mt", type=str, nargs="+", help="A list of acceptable media types for the virtual media"
)
eject_argget = subparsers.add_parser("eject", help="Ejects virtual media from a system")
eject_argget.add_argument(
    "--id", "-i", type=str, required=True, help="The identifier of the virtual media instance to eject"
)
args = argget.parse_args()

if args.debug:
    log_file = "rf_virtual_media-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_virtual_media Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    if args.command == "insert":
        print("Inserting '{}'".format(args.image))
        redfish_utilities.insert_virtual_media(
            redfish_obj, args.image, args.system, args.id, args.mediatypes, args.notinserted, args.writable
        )
    elif args.command == "eject":
        print("Ejecting '{}'".format(args.id))
        redfish_utilities.eject_virtual_media(redfish_obj, args.id, args.system)
    else:
        virtual_media = redfish_utilities.get_virtual_media(redfish_obj, args.system)
        redfish_utilities.print_virtual_media(virtual_media)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Power Equipment

File : rf_power_equipment.py

Brief : This script uses the redfish_utilities module to manage power equipment
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to manage power equipment")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
subparsers = argget.add_subparsers(dest="command")
list_argget = subparsers.add_parser("list", help="Displays a list of the available power equipment")
status_argget = subparsers.add_parser("status", help="Displays the status of an instance of power equipment")
status_argget.add_argument(
    "--type",
    "-t",
    type=redfish_utilities.power_equipment_types,
    required=True,
    help="The type of power equipment to get",
    choices=redfish_utilities.power_equipment_types,
)
status_argget.add_argument("--equipment", "-pe", type=str, help="The identifier of the power equipment to get")
outletsummary_argget = subparsers.add_parser(
    "outlets", help="Displays the outlet summary of an instance of power equipment"
)
outletsummary_argget.add_argument(
    "--type",
    "-t",
    type=redfish_utilities.power_equipment_types,
    required=True,
    help="The type of power equipment to get",
    choices=redfish_utilities.power_equipment_types,
)
outletsummary_argget.add_argument("--equipment", "-pe", type=str, help="The identifier of the power equipment to get")
outlet_info_argget = subparsers.add_parser(
    "outletinfo", help="Displays the status of an outlet for an instance of power equipment"
)
outlet_info_argget.add_argument(
    "--type",
    "-t",
    type=redfish_utilities.power_equipment_types,
    required=True,
    help="The type of power equipment to get",
    choices=redfish_utilities.power_equipment_types,
)
outlet_info_argget.add_argument("--equipment", "-pe", type=str, help="The identifier of the power equipment to get")
outlet_info_argget.add_argument("--outlet", "-o", type=str, help="The identifier of the outlet to get")
mains_info_argget = subparsers.add_parser(
    "mainsinfo", help="Displays the status of a mains circuit for an instance of power equipment"
)
mains_info_argget.add_argument(
    "--type",
    "-t",
    type=redfish_utilities.power_equipment_types,
    required=True,
    help="The type of power equipment to get",
    choices=redfish_utilities.power_equipment_types,
)
mains_info_argget.add_argument("--equipment", "-pe", type=str, help="The identifier of the power equipment to get")
mains_info_argget.add_argument("--mains", "-m", type=str, help="The identifier of the mains circuit to get")
branch_info_argget = subparsers.add_parser(
    "branchinfo", help="Displays the status of a branch circuit for an instance of power equipment"
)
branch_info_argget.add_argument(
    "--type",
    "-t",
    type=redfish_utilities.power_equipment_types,
    required=True,
    help="The type of power equipment to get",
    choices=redfish_utilities.power_equipment_types,
)
branch_info_argget.add_argument("--equipment", "-pe", type=str, help="The identifier of the power equipment to get")
branch_info_argget.add_argument("--branch", "-b", type=str, help="The identifier of the branch circuit to get")
args = argget.parse_args()

if args.debug:
    log_file = "rf_power_equipment-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_power_equipment Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    if args.command == "status":
        power_equipment = redfish_utilities.get_power_equipment(
            redfish_obj,
            args.type,
            args.equipment,
            get_metrics=True,
            get_mains=True,
            get_branches=True,
            get_feeders=True,
            get_subfeeds=True,
        )
        redfish_utilities.print_power_equipment(power_equipment)
        circuit_types = [
            redfish_utilities.power_equipment_electrical_types.MAINS,
            redfish_utilities.power_equipment_electrical_types.BRANCH,
            redfish_utilities.power_equipment_electrical_types.FEEDER,
            redfish_utilities.power_equipment_electrical_types.SUBFEED,
        ]
        for circuit_type in circuit_types:
            redfish_utilities.print_power_equipment_electrical_summary(power_equipment, circuit_type, False)
    elif args.command == "outlets":
        power_equipment = redfish_utilities.get_power_equipment(
            redfish_obj, args.type, args.equipment, get_outlets=True
        )
        redfish_utilities.print_power_equipment_electrical_summary(
            power_equipment, redfish_utilities.power_equipment_electrical_types.OUTLET, True
        )
    elif args.command == "outletinfo":
        outlet = redfish_utilities.get_power_equipment_electrical(
            redfish_obj,
            args.type,
            redfish_utilities.power_equipment_electrical_types.OUTLET,
            args.equipment,
            args.outlet,
        )
        redfish_utilities.print_power_equipment_electrical(outlet)
    elif args.command == "mainsinfo":
        mains = redfish_utilities.get_power_equipment_electrical(
            redfish_obj, args.type, redfish_utilities.power_equipment_electrical_types.MAINS, args.equipment, args.mains
        )
        redfish_utilities.print_power_equipment_electrical(mains)
    elif args.command == "branchinfo":
        mains = redfish_utilities.get_power_equipment_electrical(
            redfish_obj,
            args.type,
            redfish_utilities.power_equipment_electrical_types.BRANCH,
            args.equipment,
            args.branch,
        )
        redfish_utilities.print_power_equipment_electrical(mains)
    else:
        power_equipment = redfish_utilities.get_power_equipment_summary(redfish_obj)
        redfish_utilities.print_power_equipment_summary(power_equipment)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Resets Module

File : resets.py

Brief : This file contains the common definitions and functionalities for
        reset operations
"""

reset_types = [
    "On",
    "ForceOff",
    "GracefulShutdown",
    "GracefulRestart",
    "ForceRestart",
    "Nmi",
    "ForceOn",
    "PushPowerButton",
    "PowerCycle",
    "Suspend",
    "Pause",
    "Resume",
]

reset_to_defaults_types = ["ResetAll", "PreserveNetworkAndUsers", "PreserveNetwork"]



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Test Event Listener

File : rf_test_event_listener.py

Brief : This script performs POST operations with event payloads to help verify
        an event listener.
"""

import argparse
import json
import requests
from datetime import datetime

event_headers = {"Content-Type": "application/json"}

event_payload = {
    "@odata.type": "#Event.v1_7_0.Event",
    "Id": "1",
    "Name": "Sample Event",
    "Context": "Sample Event for Listener",
    "Events": [
        {
            "EventType": "Other",
            "EventId": "1",
            "Severity": "OK",
            "MessageSeverity": "OK",
            "Message": "Test message.",
            "MessageId": "Resource.1.3.TestMessage",
        }
    ],
}

argget = argparse.ArgumentParser(description="A tool to help verify a Redfish event listener")
argget.add_argument(
    "--listener", "-l", type=str, required=True, help="The absolute URI of the Redfish event listener (with scheme)"
)
argget.add_argument(
    "--file",
    "-file",
    type=str,
    help="The filepath to a JSON file containing the event payload; if this argument is specified, all other arguments controlling the event data is ignored",
)
argget.add_argument("--id", "-id", type=str, help="The value to specify in the Id property of the event")
argget.add_argument("--name", "-name", type=str, help="The value to specify in the Name property of the event")
argget.add_argument("--context", "-context", type=str, help="The value to specify in the Context property of the event")
argget.add_argument(
    "--eventtype", "-eventtype", type=str, help="The value to specify in the EventType property of the event"
)
argget.add_argument("--eventid", "-eventid", type=str, help="The value to specify in the EventId property of the event")
argget.add_argument(
    "--severity", "-severity", type=str, help="The value to specify in the Severity property of the event"
)
argget.add_argument("--message", "-message", type=str, help="The value to specify in the Message property of the event")
argget.add_argument(
    "--messageid", "-messageid", type=str, help="The value to specify in the MessageId property of the event"
)
argget.add_argument(
    "--timestamp", "-timestamp", type=str, help="The value to specify in the EventTimestamp property of the event"
)
argget.add_argument(
    "--header",
    "-header",
    type=str,
    nargs=2,
    metavar=("name", "value"),
    action="append",
    help="Name-value pairs of HTTP headers to provide with the request",
)
args = argget.parse_args()

# Update the event payload based on the specified arguments
if args.file:
    with open(args.file) as json_file:
        event_payload = json.load(json_file)
else:
    if args.id:
        event_payload["Id"] = args.id
    if args.name:
        event_payload["Name"] = args.name
    if args.context:
        event_payload["Context"] = args.context
    if args.eventtype:
        event_payload["Events"][0]["EventType"] = args.eventtype
    if args.eventid:
        event_payload["Events"][0]["EventId"] = args.eventid
    if args.severity:
        event_payload["Events"][0]["Severity"] = args.severity
        event_payload["Events"][0]["MessageSeverity"] = args.severity
    if args.message:
        event_payload["Events"][0]["Message"] = args.message
    if args.messageid:
        event_payload["Events"][0]["MessageId"] = args.messageid
    if args.timestamp:
        event_payload["Events"][0]["EventTimestamp"] = args.timestamp
    else:
        event_payload["Events"][0]["EventTimestamp"] = datetime.now().replace(microsecond=0).astimezone().isoformat()

# Update the HTTP headers based on the specified arguments
if args.header:
    for header in args.header:
        event_headers[header[0]] = header[1]

# Send the request
response = requests.post(args.listener, json=event_payload, headers=event_headers, timeout=15, verify=False)
print("Listener responded with {} {}".format(response.status_code, response.reason))



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Logs Module

File : logs.py

Brief : This file contains the definitions and functionalities for interacting
        with the log service for a given Redfish service
"""

from .messages import verify_response
from enum import Enum


class RedfishLogServiceNotFoundError(Exception):
    """
    Raised when a matching log service cannot be found
    """

    pass


class RedfishClearLogNotFoundError(Exception):
    """
    Raised when a log service does not contain the clear log action
    """

    pass


class RedfishCollectDiagnosticDataNotFoundError(Exception):
    """
    Raised when a log service does not contain the collect diagnostic data action
    """

    pass


class RedfishDiagnosticDataNotFoundError(Exception):
    """
    Raised when the requested diagnostic data cannot be found
    """

    pass


class log_container(Enum):
    """
    Types of resources that contain log services
    """

    MANAGER = "Managers"
    CHASSIS = "Chassis"
    SYSTEM = "Systems"


class diagnostic_data_types(Enum):
    """
    Types of diagnostic data that can be requested
    """

    MANAGER = "Manager"
    PRE_OS = "PreOS"
    OS = "OS"
    OEM = "OEM"

    def __str__(self):
        return self.value


def get_log_entries(context, container_type=log_container.MANAGER, container_id=None, log_service_id=None):
    """
    Finds the log entries of a log service matching the given ID

    Args:
        context: The Redfish client object with an open session
        container_type: The type of resource containing the log service (manager, system, or chassis)
        container_id: The container instance with the log service; if None, perform on the only container
        log_service_id: The log service with the logs; if None, perform on the only log service

    Returns:
        An array of log entries
    """

    log_service = get_log_service(context, container_type, container_id, log_service_id)

    # Read in the log entries
    log_entries = []
    log_entry_col = context.get(log_service.dict["Entries"]["@odata.id"])
    log_entries.extend(log_entry_col.dict["Members"])

    # If a next link is provided, iterate over it and add to the log entry list
    while "Members@odata.nextLink" in log_entry_col.dict:
        log_entry_col = context.get(log_entry_col.dict["Members@odata.nextLink"])
        log_entries.extend(log_entry_col.dict["Members"])

    return log_entries


def print_log_entries(log_entries, details=False):
    """
    Prints a set of log entries in a table

    Args:
        log_entries: The log entries to print
        details: Flag indicating if details should be displayed
    """

    # Set up templates
    entry_line_format = "  {:5s} | {:25s} | {}"
    detail_line_format = "  {:33s} | {}: {}"
    detail_list = [
        "Severity",
        "EntryType",
        "OemRecordFormat",
        "EntryCode",
        "OemLogEntryCode",
        "SensorType",
        "OemSensorType",
        "GeneratorId",
        "SensorNumber",
        "EventType",
        "EventId",
        "EventGroupId",
        "MessageId",
        "MessageArgs",
    ]
    print(entry_line_format.format("Id", "Timestamp", "Message"))

    # Go through each entry and print the info
    for entry in log_entries:
        timestamp_property = "Created"
        if "EventTimestamp" in entry:
            timestamp_property = "EventTimestamp"
        print(
            entry_line_format.format(
                entry["Id"],
                entry.get(timestamp_property, "Unknown"),
                entry["Message"].replace("\n", "; "),
            )
        )
        if details:
            for detail in detail_list:
                if detail in entry:
                    print(detail_line_format.format("", detail, entry[detail]))


def clear_log_entries(context, container_type=log_container.MANAGER, container_id=None, log_service_id=None):
    """
    Clears the log entries of a log service matching the given ID

    Args:
        context: The Redfish client object with an open session
        container_type: The type of resource containing the log service (manager, system, or chassis)
        container_id: The container instance with the log service; if None, perform on the only container
        log_service_id: The log service with the logs; if None, perform on the only log service

    Returns:
        The response of the action
    """

    log_service = get_log_service(context, container_type, container_id, log_service_id)

    # Find the ClearLog action
    if "Actions" not in log_service.dict:
        raise RedfishClearLogNotFoundError("Log service does not support ClearLog")
    if "#LogService.ClearLog" not in log_service.dict["Actions"]:
        raise RedfishClearLogNotFoundError("Log service does not support ClearLog")
    clear_uri = log_service.dict["Actions"]["#LogService.ClearLog"]["target"]

    # Clear the log
    response = context.post(clear_uri, body={})
    verify_response(response)
    return response


def collect_diagnostic_data(
    context,
    container_type=log_container.MANAGER,
    container_id=None,
    log_service_id=None,
    diagnostic_data_type=None,
    oem_data_type=None,
):
    """
    Performs diagnostic data collection of a log service matching the given ID

    Args:
        context: The Redfish client object with an open session
        container_type: The type of resource containing the log service (manager, system, or chassis)
        container_id: The container instance with the log service; if None, perform on the only container
        log_service_id: The log service with the logs; if None, perform on the only log service
        diagnostic_data_type: The type of diagnostic data to collect (manager, pre OS, OS, OEM)
        oem_data_type: The type of OEM data to collect

    Returns:
        The response of the action
    """

    log_service = get_log_service(context, container_type, container_id, log_service_id)

    # Find the ClearLog action
    if "Actions" not in log_service.dict:
        raise RedfishCollectDiagnosticDataNotFoundError("Log service does not support CollectDiagnosticData")
    if "#LogService.CollectDiagnosticData" not in log_service.dict["Actions"]:
        raise RedfishCollectDiagnosticDataNotFoundError("Log service does not support CollectDiagnosticData")
    collect_uri = log_service.dict["Actions"]["#LogService.CollectDiagnosticData"]["target"]

    # Collect diagnostic data
    if diagnostic_data_type is None:
        diagnostic_data_type = diagnostic_data_types.MANAGER
    payload = {"DiagnosticDataType": diagnostic_data_type.value}
    if oem_data_type is not None and diagnostic_data_type == diagnostic_data_types.OEM:
        # Only include OEMDiagnosticDataType if the OEM type is requested
        payload["OEMDiagnosticDataType"] = oem_data_type
    response = context.post(collect_uri, body=payload)
    verify_response(response)
    return response


def download_diagnostic_data(context, collect_response):
    """
    Downloads the diagnostic data based on the response from the collect action

    Args:
        context: The Redfish client object with an open session
        collect_response: The response object from the collect diagnostic data action

    Returns:
        The name of the file downloaded from the service
        An array of bytes containing the file contents
    """

    # Follow the Location header to the log entry
    entry_uri = collect_response.getheader("Location")
    if entry_uri is None:
        raise RedfishDiagnosticDataNotFoundError(
            "The response to collecting diagnostic data does not contain a location for the data"
        )

    # Get the log entry
    response = context.get(entry_uri)
    if "AdditionalDataURI" not in response.dict:
        raise RedfishDiagnosticDataNotFoundError(
            "The log entry for the collected data does not contain an additional data link"
        )

    # Download the file
    filename = response.dict["AdditionalDataURI"].split("/")[-1]
    response = context.get(response.dict["AdditionalDataURI"])
    return (
        filename,
        response._http_response.content,
    )  # TODO: May need to push support in python-redfish-library to have a proper method of getting raw content


def get_log_service(context, container_type=log_container.MANAGER, container_id=None, log_service_id=None):
    """
    Finds a log service matching the given ID and returns its resource

    Args:
        context: The Redfish client object with an open session
        container_type: The type of resource containing the log service (manager, system, or chassis)
        container_id: The container instance to locate; if None, perform on the only container
        log_service_id: The log service to locate; if None, perform on the only log service

    Returns:
        The log service resource
    """

    # Get the Service Root to find the resource collection
    service_root = context.get("/redfish/v1/")
    if container_type.value not in service_root.dict:
        # No resource collection
        raise RedfishLogServiceNotFoundError("Service does not contain a {} collection".format(container_type.value))

    # Get the resource collection and iterate through its collection to find the matching container instance
    avail_resources = []
    resource_col = context.get(service_root.dict[container_type.value]["@odata.id"])
    if container_id is None:
        if len(resource_col.dict["Members"]) == 1:
            resource = context.get(resource_col.dict["Members"][0]["@odata.id"])
            container_id = resource.dict["Id"]
        else:
            for resource_member in resource_col.dict["Members"]:
                resource = context.get(resource_member["@odata.id"])
                avail_resources.append(resource.dict["Id"])
            raise RedfishLogServiceNotFoundError(
                "Service does not contain exactly one resource in {}; a target container needs to be specified: {}".format(
                    container_type.value, ", ".join(avail_resources)
                )
            )
    else:
        container_found = False
        for resource_member in resource_col.dict["Members"]:
            resource = context.get(resource_member["@odata.id"])
            avail_resources.append(resource.dict["Id"])
            if resource.dict["Id"] == container_id:
                container_found = True
                break
        if not container_found:
            raise RedfishLogServiceNotFoundError(
                "Service does not contain a resource in {} called {}; valid resources: {}".format(
                    container_type.value, container_id, ", ".join(avail_resources)
                )
            )

    # Go through the container to find the log service collection
    if "LogServices" not in resource.dict:
        # No log services
        raise RedfishLogServiceNotFoundError("{} does not contain a log service collection".format(container_id))

    # Get the log service collection and iterate through its collection
    avail_logs = []
    log_serv_col = context.get(resource.dict["LogServices"]["@odata.id"])
    if log_service_id is None:
        if len(log_serv_col.dict["Members"]) == 1:
            return context.get(log_serv_col.dict["Members"][0]["@odata.id"])
        else:
            for log_serv_member in log_serv_col.dict["Members"]:
                log_serv = context.get(log_serv_member["@odata.id"])
                avail_logs.append(log_serv.dict["Id"])
            raise RedfishLogServiceNotFoundError(
                "{} does not contain exactly one log service; a target log service needs to be specified: {}".format(
                    container_id, ", ".join(avail_logs)
                )
            )
    else:
        for log_serv_member in log_serv_col.dict["Members"]:
            log_serv = context.get(log_serv_member["@odata.id"])
            avail_logs.append(log_serv.dict["Id"])
            if log_serv.dict["Id"] == log_service_id:
                return log_serv

    raise RedfishLogServiceNotFoundError(
        "{} does not contain a log service called {}; valid log services: {}".format(
            container_id, log_service_id, ", ".join(avail_logs)
        )
    )



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Update

File : rf_update.py

Brief : This script uses the redfish_utilities module to perform updates
"""

import argparse
import datetime
import logging
import os
import re
import redfish
import redfish_utilities
import shutil
import socket
import sys
import threading
import time
import traceback
from redfish.messages import RedfishPasswordChangeRequiredError

if sys.version_info > (3,):
    import http.server
else:
    import SimpleHTTPServer
    import SocketServer

WEB_SERVER_PORT = 8888
WEB_SERVER_FOLDER = "rf_update_server"


def local_web_server(filepath):
    """
    Creates a web server to host the image file

    Args:
        filepath: The filepath for the image
    """

    # Create a temporary folder and move the file to the folder
    try:
        shutil.rmtree(WEB_SERVER_FOLDER)
    except Exception:
        pass
    os.mkdir(WEB_SERVER_FOLDER)
    shutil.copy(filepath, WEB_SERVER_FOLDER)
    os.chdir(WEB_SERVER_FOLDER)

    # Create a web server and host it out of the temporary folder
    if sys.version_info > (3,):
        handler = http.server.SimpleHTTPRequestHandler
        httpd = http.server.HTTPServer(("", WEB_SERVER_PORT), handler)
    else:
        handler = SimpleHTTPServer.SimpleHTTPRequestHandler
        httpd = SocketServer.TCPServer(("", WEB_SERVER_PORT), handler)
    httpd.serve_forever()

    return


def print_error_payload(response):
    """
    Prints an error payload, which can also be used for action responses

    Args:
        response: The response to print
    """

    try:
        print(redfish_utilities.get_error_messages(response))
    except Exception:
        # No response body
        if response.status >= 400:
            print("Failed")
        else:
            print("Success")


# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to perform an update with a Redfish service")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument("--image", "-i", type=str, required=True, help="The URI or filepath of the image")
argget.add_argument("--target", "-t", type=str, help="The target resource to apply the image")
argget.add_argument(
    "--applytime",
    "-at",
    type=redfish_utilities.operation_apply_times,
    help="The apply time for the update",
    choices=redfish_utilities.operation_apply_times,
)
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
args = argget.parse_args()

if args.debug:
    log_file = "rf_update-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_update Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

start_path = os.getcwd()
targets = None
if args.target is not None:
    targets = [args.target]
exit_code = 0
try:
    # Determine what path to use to perform the update
    update_service = redfish_utilities.get_update_service(redfish_obj)
    if os.path.isfile(args.image):
        # Local image; see if we can push the image directly
        if "MultipartHttpPushUri" in update_service.dict:
            # Perform a multipart push update
            print(
                "Pushing the image to the service directly; depending on the size of the image, this can take a few minutes..."
            )
            response = redfish_utilities.multipart_push_update(
                redfish_obj, args.image, targets=targets, apply_time=args.applytime
            )
        else:
            # Host a local web server and perform a SimpleUpdate for the local image
            web_server_thread = threading.Thread(target=local_web_server, args=(args.image,))
            web_server_thread.setDaemon(True)
            web_server_thread.start()
            # Wait for the server to start up
            time.sleep(3)

            # Build the proper image URI for the call based on how the web server will be hosting it
            # TODO: Find a better way of getting your own IP address
            # socket.gethostbyname( socket.gethostname() ) returns 127.0.0.1 on many systems
            # This will open a socket with the target, and pulls the address of the socket
            groups = re.search("^(https?)://([^:]+)(:(\\d+))?$", args.rhost)
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            remote_port = groups.group(4)
            if remote_port is None:
                remote_port = "80"
                if groups.group(1) == "https":
                    remote_port = "443"
            s.connect((groups.group(2), int(remote_port)))
            image_uri = "http://{}:{}/{}".format(
                s.getsockname()[0], WEB_SERVER_PORT, args.image.rsplit(os.path.sep, 1)[-1]
            )
            s.close()
            response = redfish_utilities.simple_update(
                redfish_obj, image_uri, targets=targets, apply_time=args.applytime
            )
    else:
        # Remote image; always use SimpleUpdate
        response = redfish_utilities.simple_update(redfish_obj, args.image, targets=targets, apply_time=args.applytime)

    # Monitor the response
    print("Update initiated...")
    response = redfish_utilities.poll_task_monitor(redfish_obj, response)

    # Display the results
    print("")
    print_error_payload(response)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out and cleanup
    os.chdir(start_path)
    try:
        shutil.rmtree(WEB_SERVER_FOLDER)
    except Exception:
        pass
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Collections Module

File : collections.py

Brief : This file contains the definitions and functionalities for performing
        operations with resource collections
"""

from .messages import verify_response


class RedfishCollectionNotFoundError(Exception):
    """
    Raised when the specified collection is not found (HTTP Status = 404)
    """

    pass


class RedfishCollectionMemberNotFoundError(Exception):
    """
    Raised when the specified member is not found (HTTP Status = 404)
    """

    pass


def get_collection_ids(context, collection_uri):
    """
    Iterates over a collection and returns the identifiers of all members

    Args:
        context: The Redfish client object with an open session
        collection_uri: The URI of the collection to process

    Returns:
        A list of identifiers of the members of the collection
    """

    # Get the collection and iterate through its collection
    avail_members = []
    collection = context.get(collection_uri)
    if collection.status == 404:
        raise RedfishCollectionNotFoundError("Service does not contain a collection at URI {}".format(collection_uri))
    verify_response(collection)
    while True:
        for member in collection.dict["Members"]:
            avail_members.append(member["@odata.id"].strip("/").split("/")[-1])
        if "Members@odata.nextLink" not in collection.dict:
            break
        collection = context.get(collection.dict["Members@odata.nextLink"])
        verify_response(collection)

    return avail_members



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Assembly

File : rf_assembly.py

Brief : This script uses the redfish_utilities module to manage assemblies
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to manage assemblies on a Redfish service")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument("--assembly", "-a", type=str, required=True, help="The URI of the target assembly")
argget.add_argument("--index", "-i", type=int, help="The target assembly index")
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
subparsers = argget.add_subparsers(dest="command")
info_argget = subparsers.add_parser("info", help="Displays information about the an assembly")
download_argget = subparsers.add_parser("download", help="Downloads assembly data to a file")
download_argget.add_argument(
    "--file", "-f", type=str, required=True, help="The file, and optional path, to save the assembly data"
)
upload_argget = subparsers.add_parser("upload", help="Uploads assembly data from a file")
upload_argget.add_argument(
    "--file", "-f", type=str, required=True, help="The file, and optional path, containing the assembly data to upload"
)
args = argget.parse_args()

if args.index and args.index < 0:
    print("rf_assembly.py: error: the assembly index cannot be negative")
    sys.exit(1)

if args.debug:
    log_file = "rf_assembly-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_assembly Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    assembly_info = redfish_utilities.get_assembly(redfish_obj, args.assembly)
    if args.command == "download":
        print("Saving data to '{}'...".format(args.file))
        redfish_utilities.download_assembly(redfish_obj, assembly_info, args.file, args.index)
    elif args.command == "upload":
        print("Writing data from '{}'...".format(args.file))
        redfish_utilities.upload_assembly(redfish_obj, assembly_info, args.file, args.index)
    else:
        redfish_utilities.print_assembly(assembly_info, args.index)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Certificates Module

File : certificates.py

Brief : This file contains the definitions and functionalities for managing
        certificates on a Redfish service
"""

from .messages import verify_response


class RedfishCertificateServiceNotFoundError(Exception):
    """
    Raised when the certificate service cannot be found
    """

    pass


class RedfishCertificateLocationsNotFoundError(Exception):
    """
    Raised when the certificate locations cannot be found
    """

    pass


class RedfishGenerateCSRActionNotFoundError(Exception):
    """
    Raised when the GenerateCSR action cannot be found
    """

    pass


class RedfishReplaceCertificateActionNotFoundError(Exception):
    """
    Raised when the ReplaceCertificate action cannot be found
    """

    pass


def get_all_certificates(context):
    """
    Collects certificate information from a Redfish service

    Args:
        context: The Redfish client object with an open session

    Returns:
        A list containing all certificates
    """

    certificate_list = []

    # Get the certificate locations
    certificate_service = get_certificate_service(context)
    if "CertificateLocations" not in certificate_service:
        raise RedfishCertificateLocationsNotFoundError("Service does not contain certificate locations")
    certificate_locations = context.get(certificate_service["CertificateLocations"]["@odata.id"])
    verify_response(certificate_locations)

    # Get each member and add it to the response list
    locations = certificate_locations.dict.get("Links", {}).get("Certificates", [])
    for certificate_ref in locations:
        certificate = context.get(certificate_ref["@odata.id"])
        verify_response(certificate)
        certificate_info = {
            "URI": certificate.dict["@odata.id"],
            "Id": certificate.dict["Id"],
            "Subject": certificate.dict.get("Subject", {}),
            "Issuer": certificate.dict.get("Issuer", {}),
            "ValidNotBefore": certificate.dict.get("ValidNotBefore"),
            "ValidNotAfter": certificate.dict.get("ValidNotAfter"),
            "KeyUsage": certificate.dict.get("KeyUsage"),
            "SerialNumber": certificate.dict.get("SerialNumber"),
            "Fingerprint": certificate.dict.get("Fingerprint"),
            "FingerprintHashAlgorithm": certificate.dict.get("FingerprintHashAlgorithm"),
            "SignatureAlgorithm": certificate.dict.get("SignatureAlgorithm"),
            "CertificateUsageTypes": certificate.dict.get("CertificateUsageTypes"),
        }
        certificate_list.append(certificate_info)

    return certificate_list


def print_certificates(certificate_list, details=False):
    """
    Prints the certificate list into a table

    Args:
        certificate_list: The certificate list to print
        details: True to print all the detailed info
    """

    # Go through each certificate
    for certificate in certificate_list:
        print("Certificate: {}".format(certificate["URI"]))
        print("  Subject: {}".format(build_identifier_string(certificate["Subject"])))
        print("  Issuer: {}".format(build_identifier_string(certificate["Issuer"])))
        print(
            "  Valid Not Before: {}, Valid Not After: {}".format(
                certificate["ValidNotBefore"], certificate["ValidNotAfter"]
            )
        )
        if details:
            if certificate["KeyUsage"] is not None:
                print("  Key Usage: {}".format(", ".join(certificate["KeyUsage"])))
            if certificate["CertificateUsageTypes"] is not None:
                print("  Certificate Usage: {}".format(", ".join(certificate["CertificateUsageTypes"])))
            if certificate["SerialNumber"] is not None:
                print("  Serial Number: {}".format(certificate["SerialNumber"]))
            if certificate["Fingerprint"] is not None:
                print("  Fingerprint: {}".format(certificate["Fingerprint"]))
            if certificate["FingerprintHashAlgorithm"] is not None:
                print("  Fingerprint Hash Algorithm: {}".format(certificate["FingerprintHashAlgorithm"]))
            if certificate["SignatureAlgorithm"] is not None:
                print("  Signature Algorithm: {}".format(certificate["SignatureAlgorithm"]))
        print("")


def get_generate_csr_info(context):
    """
    Finds information about the support for generating CSRs

    Args:
        context: The Redfish client object with an open session

    Returns:
        The URI of the GenerateCSR action
        A list of parameter requirements from the action info
    """

    # Check that there is a GenerateCSR action
    certificate_service = get_certificate_service(context)
    if "Actions" not in certificate_service:
        raise RedfishGenerateCSRActionNotFoundError("Service does not support the GenerateCSR action")
    if "#CertificateService.GenerateCSR" not in certificate_service["Actions"]:
        raise RedfishGenerateCSRActionNotFoundError("Service does not support the GenerateCSR action")

    # Extract the info about the GenerateCSR action
    generate_csr_action = certificate_service["Actions"]["#CertificateService.GenerateCSR"]
    generate_csr_uri = generate_csr_action["target"]

    if "@Redfish.ActionInfo" not in generate_csr_action:
        # No action info; due to the action's complexity, don't try to produce a default set of parameters
        generate_csr_parameters = None
    else:
        # Get the action info and its parameter listing
        action_info = context.get(generate_csr_action["@Redfish.ActionInfo"])
        generate_csr_parameters = action_info.dict["Parameters"]

    return generate_csr_uri, generate_csr_parameters


def generate_csr(
    context,
    common_name,
    organization,
    organizational_unit,
    city,
    state,
    country,
    cert_col,
    email=None,
    key_pair_alg=None,
    key_bit_len=None,
    key_curve_id=None,
):
    """
    Generates a certificate signing request

    Args:
        context: The Redfish client object with an open session
        common_name: The common name of the component to secure
        organization: The name of the organization making the request
        organizational_unit: The name of the unit in the organization making the request
        city: The city or locality of the organization making the request
        state: The state, province, or region of the organization making the request
        country: The two-letter country code of the organization making the request
        cert_col: The URI of the certificate collection where the signed certificate will be installed
        email: The email address of the contact within the organization making the request
        key_pair_alg: The type of key-pair for use with signing algorithms
        key_bit_len: The length of the key, in bits, if the key pair algorithm supports key size
        key_curve_id: The curve ID to use with the key if the key pair algorithm supports curves

    Returns:
        The response of the action
    """

    # Locate the GenerateCSR action
    generate_csr_uri, generate_csr_parameters = get_generate_csr_info(context)

    # Build the payload
    payload = {
        "CertificateCollection": {"@odata.id": cert_col},
        "CommonName": common_name,
        "Organization": organization,
        "OrganizationalUnit": organizational_unit,
        "City": city,
        "State": state,
        "Country": country,
    }
    if email is not None:
        payload["Email"] = email
    if key_pair_alg is not None:
        payload["KeyPairAlgorithm"] = key_pair_alg
    if key_bit_len is not None:
        payload["KeyBitLength"] = key_bit_len
    if key_curve_id is not None:
        payload["KeyCurveId"] = key_curve_id

    # Generate a CSR
    response = context.post(generate_csr_uri, body=payload)
    verify_response(response)
    return response


def install_certificate(context, destination, cert_file, key_file=None):
    """
    Replaces an existing certificate with a new certificate

    Args:
        context: The Redfish client object with an open session
        destination: The installation URI of the certificate; either a certificate collection to insert, or an existing certificate to replace
        cert_file: The file, and optional path, of the certificate to install
        key_file: The file, and optional path, of the private key for the certificate to install

    Returns:
        The response of the operation
    """

    # Read the certificate and determine its type
    with open(cert_file, "r") as file_handle:
        cert_string = file_handle.read()
    cert_type = "PEM"
    if "BEGIN PKCS7" in cert_string:
        cert_type = "PKCS7"
    elif cert_string.count("BEGIN CERTIFICATE") > 1:
        cert_type = "PEMchain"

    # Read the key if needed, and prepend it to the certificate
    if key_file is not None:
        with open(key_file, "r") as file_handle:
            key_string = file_handle.read()
        cert_string = key_string + cert_string

    # Build the payload
    payload = {
        "CertificateUri": {"@odata.id": destination},
        "CertificateString": cert_string,
        "CertificateType": cert_type,
    }

    # Check if the referenced destination is a certificate collection or individual certificate
    dest_response = context.get(destination)
    verify_response(dest_response)
    if dest_response.dict["@odata.type"] == "#CertificateCollection.CertificateCollection":
        # Certificate collection; just perform a POST operation on the collection
        payload.pop("CertificateUri")
    else:
        # Individual certificate; perform a replacement operation from the certificate service

        # Locate the ReplaceCertificate action
        certificate_service = get_certificate_service(context)
        if "Actions" not in certificate_service:
            raise RedfishReplaceCertificateActionNotFoundError("Service does not support the ReplaceCertificate action")
        if "#CertificateService.ReplaceCertificate" not in certificate_service["Actions"]:
            raise RedfishReplaceCertificateActionNotFoundError("Service does not support the ReplaceCertificate action")
        destination = certificate_service["Actions"]["#CertificateService.ReplaceCertificate"]["target"]

    # Install the certificate
    response = context.post(destination, body=payload)
    verify_response(response)
    return response


def delete_certificate(context, certificate):
    """
    Replaces an existing certificate with a new certificate

    Args:
        context: The Redfish client object with an open session
        certificate: The URI of the certificate to delete

    Returns:
        The response of the operation
    """

    # Delete the certificate
    response = context.delete(certificate)
    verify_response(response)
    return response


def get_certificate_service(context):
    """
    Collects the certificate service information from a Redfish service

    Args:
        context: The Redfish client object with an open session

    Returns:
        An object containing information about the certificate service
    """

    # Get the service root to find the certificate service
    service_root = context.get("/redfish/v1/")
    if "CertificateService" not in service_root.dict:
        # No event service
        raise RedfishCertificateServiceNotFoundError("Service does not contain a certificate service")

    # Get the certificate service
    certificate_service = context.get(service_root.dict["CertificateService"]["@odata.id"])
    verify_response(certificate_service)
    return certificate_service.dict


def build_identifier_string(identifier):
    """
    Creates an identifier string for a subject or issuer object

    Args:
        identifier: The identifier object to parse

    Returns:
        A string containing the identifier info
    """

    field_map = {
        "CommonName": "CN",
        "Organization": "O",
        "OrganizationalUnit": "OU",
        "Country": "C",
        "State": "ST",
        "City": "L",
        "Email": "emailAddress",
    }

    strings = []
    for map in field_map:
        if map in identifier and identifier[map] is not None:
            strings.append("{}={}".format(field_map[map], identifier[map]))
    return ", ".join(strings)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
PDU Module

File : power_equipment.py

Brief : This file contains the definitions and functionalities for interacting
        with the power equipment for a given Redfish service
"""

from .collections import get_collection_ids
from .messages import verify_response
from enum import Enum


class RedfishPowerEquipmentNotFoundError(Exception):
    """
    Raised when the requested power equipment is not represented in the service
    """

    pass


class RedfishPowerEquipmentElectricalNotFoundError(Exception):
    """
    Raised when the requested power equipment does not contain the desired electrical component
    """

    pass


class power_equipment_types(Enum):
    """
    Types of power equipment that can be managed
    """

    FLOOR_PDU = "FloorPDU"
    RACK_PDU = "RackPDU"
    SWITCHGEAR = "Switchgear"
    TRANSFER_SWITCH = "TransferSwitch"
    POWER_SHELF = "PowerShelf"
    ELECTRICAL_BUS = "ElectricalBus"

    def __str__(self):
        return self.value

    @property
    def plural(self):
        if self.value == "FloorPDU" or self.value == "RackPDU":
            return self.value + "s"
        if self.value == "TransferSwitch" or self.value == "ElectricalBus":
            return self.value + "es"
        if self.value == "PowerShelf":
            return "PowerShelves"
        return self.value


class power_equipment_electrical_types(Enum):
    """
    Types of electrical components found on power equipment
    """

    OUTLET = "Outlet"
    MAINS = "Mains"
    BRANCH = "Branch"
    FEEDER = "Feeder"
    SUBFEED = "Subfeed"

    def __str__(self):
        return self.value

    @property
    def plural(self):
        if self.value == "Mains":
            return self.value
        if self.value == "Branch":
            return self.value + "es"
        return self.value + "s"


voltage_type_strings = {
    "AC100To127V": "100 V to 127 V AC",
    "AC100To240V": "100 V to 240 V AC",
    "AC100To277V": "100 V to 277 V AC",
    "AC120V": "120 V AC",
    "AC200To240V": "200 V to 240 V AC",
    "AC200To277V": "200 V to 277 V AC",
    "AC208V": "208 V AC",
    "AC230V": "230 V AC",
    "AC240V": "240 V AC",
    "AC240AndDC380V": "240 V AC/380 V DC",
    "AC277V": "277 V AC",
    "AC277AndDC380V": "277 V AC/380 V DC",
    "AC400V": "400 V AC",
    "AC480V": "480 V AC",
    "DC48V": "48 V DC",
    "DC240V": "240 V DC",
    "DC380V": "380 V DC",
    "DCNeg48V": "-48 V DC",
}

phase_wiring_type_strings = {
    "OnePhase3Wire": "1 Phase, 3 Wire",
    "TwoPhase3Wire": "2 Phase, 3 Wire",
    "OneOrTwoPhase3Wire": "1 or 2 Phase, 3 Wire",
    "TwoPhase4Wire": "2 Phase, 4 Wire",
    "ThreePhase4Wire": "3 Phase, 4 Wire",
    "ThreePhase5Wire": "3 Phase, 5 Wire",
}

receptical_type_strings = {
    "BS_1363_Type_G": "BS 1363 Type G (250V; 13A)",
    "BusConnection": "Electrical bus connection",
    "CEE_7_Type_E": "CEE 7/7 Type E (250V; 16A)",
    "CEE_7_Type_F": "CEE 7/7 Type F (250V; 16A)",
    "IEC_60320_C13": "IEC C13 (250V; 10A or 15A)",
    "IEC_60320_C19": "IEC C19 (250V; 16A or 20A)",
    "NEMA_5_15R": "NEMA 5-15R (120V; 15A)",
    "NEMA_5_20R": "NEMA 5-20R (120V; 20A)",
    "NEMA_L5_20R": "NEMA L5-20R (120V; 20A)",
    "NEMA_L5_30R": "NEMA L5-30R (120V; 30A)",
    "NEMA_L6_20R": "NEMA L6-20R (250V; 20A)",
    "NEMA_L6_30R": "NEMA L6-30R (250V; 30A)",
    "SEV_1011_TYPE_12": "SEV 1011 Type 12 (250V; 10A)",
    "SEV_1011_TYPE_23": "SEV 1011 Type 23 (250V; 16A)",
}

plug_type_strings = {
    "California_CS8265": "California Standard CS8265 (Single-phase 250V; 50A; 2P3W)",
    "California_CS8365": "California Standard CS8365 (Three-phase 250V; 50A; 3P4W)",
    "Field_208V_3P4W_60A": "Field-wired; Three-phase 200-250V; 60A; 3P4W",
    "Field_400V_3P5W_32A": "Field-wired; Three-phase 200-240/346-415V; 32A; 3P5W",
    "IEC_60309_316P6": "IEC 60309 316P6 (Single-phase 200-250V; 16A; 1P3W; Blue, 6-hour)",
    "IEC_60309_332P6": "IEC 60309 332P6 (Single-phase 200-250V; 32A; 1P3W; Blue, 6-hour)",
    "IEC_60309_363P6": "IEC 60309 363P6 (Single-phase 200-250V; 63A; 1P3W; Blue, 6-hour)",
    "IEC_60309_460P9": "IEC 60309 460P9 (Three-phase 200-250V; 60A; 3P4W; Blue; 9-hour)",
    "IEC_60309_516P6": "IEC 60309 516P6 (Three-phase 200-240/346-415V; 16A; 3P5W; Red; 6-hour)",
    "IEC_60309_532P6": "IEC 60309 532P6 (Three-phase 200-240/346-415V; 32A; 3P5W; Red; 6-hour)",
    "IEC_60309_560P9": "IEC 60309 560P9 (Three-phase 120-144/208-250V; 60A; 3P5W; Blue; 9-hour)",
    "IEC_60309_563P6": "IEC 60309 563P6 (Three-phase 200-240/346-415V; 63A; 3P5W; Red; 6-hour)",
    "IEC_60320_C14": "IEC C14 (Single-phase 250V; 10A; 1P3W)",
    "IEC_60320_C20": "IEC C20 (Single-phase 250V; 16A; 1P3W)",
    "NEMA_5_15P": "NEMA 5-15P (Single-phase 125V; 15A; 1P3W)",
    "NEMA_5_20P": "NEMA 5-20P (Single-phase 125V; 20A; 1P3W)",
    "NEMA_6_15P": "NEMA 6-15P (Single-phase 250V; 15A; 2P3W)",
    "NEMA_6_20P": "NEMA 6-20P (Single-phase 250V; 20A; 2P3W)",
    "NEMA_L14_20P": "NEMA L14-20P (Split-phase 125/250V; 20A; 2P4W)",
    "NEMA_L14_30P": "NEMA L14-30P (Split-phase 125/250V; 30A; 2P4W)",
    "NEMA_L15_20P": "NEMA L15-20P (Three-phase 250V; 20A; 3P4W)",
    "NEMA_L15_30P": "NEMA L15-30P (Three-phase 250V; 30A; 3P4W)",
    "NEMA_L21_20P": "NEMA L21-20P (Three-phase 120/208V; 20A; 3P5W)",
    "NEMA_L21_30P": "NEMA L21-30P (Three-phase 120/208V; 30A; 3P5W)",
    "NEMA_L22_20P": "NEMA L22-20P (Three-phase 277/480V; 20A; 3P5W)",
    "NEMA_L22_30P": "NEMA L22-30P (Three-phase 277/480V; 30A; 3P5W)",
    "NEMA_L5_15P": "NEMA L5-15P (Single-phase 125V; 15A; 1P3W)",
    "NEMA_L5_20P": "NEMA L5-20P (Single-phase 125V; 20A; 1P3W)",
    "NEMA_L5_30P": "NEMA L5-30P (Single-phase 125V; 30A; 1P3W)",
    "NEMA_L6_15P": "NEMA L6-15P (Single-phase 250V; 15A; 2P3W)",
    "NEMA_L6_20P": "NEMA L6-20P (Single-phase 250V; 20A; 2P3W)",
    "NEMA_L6_30P": "NEMA L6-30P (Single-phase 250V; 30A; 2P3W)",
}

line_measurement_strings = {
    "Line1ToLine2": "L1-L2",
    "Line1ToNeutral": "L1-N",
    "Line2ToLine3": "L2-L3",
    "Line2ToNeutral": "L2-N",
    "Line3ToLine1": "L3-L1",
    "Line3ToNeutral": "L3-N",
    "Line1": "L1",
    "Line2": "L2",
    "Line3": "L3",
    "Neutral": "N",
}


def get_power_equipment_ids(context):
    """
    Finds the power equipment and returns all power equipment identifiers

    Args:
        context: The Redfish client object with an open session

    Returns:
        A dictionary containing lists of identifiers for each type of power equipment
    """

    # Get the service root to find the power equipment
    service_root = context.get("/redfish/v1/")
    if "PowerEquipment" not in service_root.dict:
        # No power equipment
        raise RedfishPowerEquipmentNotFoundError("The service does not contain any power equipment")

    # Get the power equipment sets
    power_equipment = context.get(service_root.dict["PowerEquipment"]["@odata.id"])
    verify_response(power_equipment)

    # Build up the lists of identifiers from the power equipment
    power_equipment_identifiers = {}
    for equip_type in power_equipment_types:
        power_equipment_identifiers[equip_type.plural] = []
        if equip_type.plural in power_equipment.dict:
            # Get the collection identifiers for this power equipment type
            power_equipment_identifiers[equip_type.plural] = get_collection_ids(
                context, power_equipment.dict[equip_type.plural]["@odata.id"]
            )

    return power_equipment_identifiers


def get_power_equipment_summary(context):
    """
    Gets a summary of all power equipment

    Args:
        context: The Redfish client object with an open session

    Returns:
        A dictionary containing lists of summary info for each type of power equipment
    """

    # Get the equipment identifiers
    power_equipment_identifiers = get_power_equipment_ids(context)

    # Build up the lists of summary info from the discovered identifiers
    power_equipment_summary = {}
    for equipment_type in power_equipment_types:
        power_equipment_summary[equipment_type.plural] = []
        if equipment_type.plural in power_equipment_identifiers:
            for equip in power_equipment_identifiers[equipment_type.plural]:
                # Get the equipment info
                equipment = get_power_equipment(context, equipment_type, equip)
                summary = {
                    "Id": equipment["Info"].dict.get("Id"),
                    "Model": equipment["Info"].dict.get("Model"),
                    "SerialNumber": equipment["Info"].dict.get("SerialNumber"),
                    "State": equipment["Info"].dict.get("Status", {}).get("State"),
                    "Health": equipment["Info"].dict.get("Status", {}).get("Health"),
                }
                power_equipment_summary[equipment_type.plural].append(summary)

    return power_equipment_summary


def print_power_equipment_summary(power_equipment_summary):
    """
    Prints the power equipment summary into a table

    Args:
        power_equipment_summary: The power equipment summary to print
    """

    summary_line_format = "  {:16s} | {:16s} | {:16s} | {:12s} | {:8s}"

    for equipment_type in power_equipment_summary:
        if len(power_equipment_summary[equipment_type]):
            print(equipment_type)
            print(summary_line_format.format("Id", "Model", "Serial Number", "State", "Health"))
            for equipment in power_equipment_summary[equipment_type]:
                print(
                    summary_line_format.format(
                        equipment["Id"] if equipment["Id"] else "",
                        equipment["Model"] if equipment["Model"] else "",
                        equipment["SerialNumber"] if equipment["SerialNumber"] else "",
                        equipment["State"] if equipment["State"] else "",
                        equipment["Health"] if equipment["Health"] else "",
                    )
                )
            print("")


def get_power_equipment(
    context,
    power_equipment_type,
    power_equipment_id=None,
    get_metrics=False,
    get_outlets=False,
    get_mains=False,
    get_branches=False,
    get_feeders=False,
    get_subfeeds=False,
):
    """
    Finds a power equipment matching the given identifier and returns its resource

    Args:
        context: The Redfish client object with an open session
        power_equipment_type: The power equipment type to get; see power_equipment_types enumeration
        power_equipment_id: The power equipment to locate; if None, perform on the only power equipment
        get_metrics: Indicates if the metrics should be returned
        get_outlets: Indicates if the outlets should be returned
        get_mains: Indicates if the mains should be returned
        get_branches: Indicates if the branches should be returned
        get_feeders: Indicates if the feeders should be returned
        get_subfeeds: Indicates if the subfeeds should be returned

    Returns:
        A dictionary containing the power equipment resource and optional subordinate resources
    """

    power_equipment_uri_pattern = "/redfish/v1/PowerEquipment/" + power_equipment_type.plural + "/{}"
    avail_power_equipment = None
    power_equipment = {"Info": None}

    # If given an identifier, get the power equipment directly
    if power_equipment_id is not None:
        power_equipment["Info"] = context.get(power_equipment_uri_pattern.format(power_equipment_id))
    # No identifier given; see if there's exactly one member
    else:
        all_power_equipment = get_power_equipment_ids(context)
        if power_equipment_type.plural not in all_power_equipment:
            raise RedfishPowerEquipmentNotFoundError(
                "The service does not contain any {}".format(power_equipment_type.plural)
            )
        avail_power_equipment = all_power_equipment[power_equipment_type.plural]
        if len(avail_power_equipment) == 1:
            power_equipment["Info"] = context.get(power_equipment_uri_pattern.format(avail_power_equipment[0]))
        else:
            raise RedfishPowerEquipmentNotFoundError(
                "The service contains multiple {}; a target power equipment needs to be specified: {}".format(
                    power_equipment_type.plural, ", ".join(avail_power_equipment)
                )
            )

    # Check the response and return the manager if the response is good
    if power_equipment["Info"].status == 404:
        if avail_power_equipment is None:
            all_equipment = get_power_equipment_ids(context)
            if power_equipment_type.plural not in all_equipment:
                raise RedfishPowerEquipmentNotFoundError(
                    "The service does not contain any {}".format(power_equipment_type.plural)
                )
            avail_equipment = all_equipment[power_equipment_type.plural]
        raise RedfishPowerEquipmentNotFoundError(
            "The service does not contain any {} called {}; valid {}: {}".format(
                power_equipment_type.plural, power_equipment_id, power_equipment_type.plural, ", ".join(avail_equipment)
            )
        )
    verify_response(power_equipment["Info"])

    # Get each of the subordinate resources requested by the caller
    resource_grabber = [
        {"Property": "Metrics", "Parameter": get_metrics, "Collection": False},
        {"Property": "Outlets", "Parameter": get_outlets, "Collection": True},
        {"Property": "Mains", "Parameter": get_mains, "Collection": True},
        {"Property": "Branches", "Parameter": get_branches, "Collection": True},
        {"Property": "Feeders", "Parameter": get_feeders, "Collection": True},
        {"Property": "Subfeeds", "Parameter": get_subfeeds, "Collection": True},
    ]
    for resource in resource_grabber:
        power_equipment[resource["Property"]] = None
        if resource["Parameter"] and resource["Property"] in power_equipment["Info"].dict:
            if resource["Collection"]:
                power_equipment[resource["Property"]] = []
                collection = get_collection_ids(
                    context, power_equipment["Info"].dict[resource["Property"]]["@odata.id"]
                )
                for member_id in collection:
                    member = context.get(
                        power_equipment["Info"].dict[resource["Property"]]["@odata.id"] + "/" + member_id
                    )
                    verify_response(member)
                    power_equipment[resource["Property"]].append(member)
            else:
                power_equipment[resource["Property"]] = context.get(
                    power_equipment["Info"].dict[resource["Property"]]["@odata.id"]
                )
                verify_response(power_equipment[resource["Property"]])

    return power_equipment


def print_power_equipment(power_equipment):
    """
    Prints the power equipment info

    Args:
        power_equipment: The power equipment info to print
    """

    power_equipment_line_format = "  {}: {}"
    print("{} {} Info".format(power_equipment["Info"].dict["EquipmentType"], power_equipment["Info"].dict["Id"]))

    print("")
    power_equipment_properties = [
        "Status",
        "EquipmentType",
        "Model",
        "Manufacturer",
        "PartNumber",
        "SerialNumber",
        "Version",
        "FirmwareVersion",
        "ProductionDate",
        "AssetTag",
        "UserLabel",
    ]
    for property in power_equipment_properties:
        if property in power_equipment["Info"].dict:
            prop_val = power_equipment["Info"].dict[property]
            if isinstance(prop_val, list):
                prop_val = ", ".join([i for i in prop_val if i is not None])
            elif property == "Status":
                prop_val = "State: {}, Health: {}".format(prop_val.get("State", "N/A"), prop_val.get("Health", "N/A"))
            print(power_equipment_line_format.format(property, prop_val))
    if power_equipment["Metrics"]:
        power_equipment_properties = [
            ("PowerWatts", "Power", "W"),
            ("EnergykWh", "Energy", "kWh"),
            ("TemperatureCelsius", "Temperature", "C"),
            ("HumidityPercent", "Humidity", "%"),
            ("PowerLoadPercent", "PowerLoad", "%"),
            ("AbsoluteHumidity", "AbsoluteHumidity", "g/m^3"),
        ]
        for property in power_equipment_properties:
            if property[0] in power_equipment["Metrics"].dict:
                if power_equipment["Metrics"].dict[property[0]]["Reading"] is None:
                    prop_val = "Unavailable"
                else:
                    prop_val = str(power_equipment["Metrics"].dict[property[0]]["Reading"]) + " " + property[2]
                print(power_equipment_line_format.format(property[1], prop_val))
    print("")


def get_power_equipment_electrical(
    context, power_equipment_type, power_equipment_electrical_type, power_equipment_id=None, electrical_id=None
):
    """
    Finds an outlet or circuit for an instance of power equipment and returns its resource

    Args:
        context: The Redfish client object with an open session
        power_equipment_type: The power equipment type to get; see power_equipment_types enumeration
        power_equipment_electrical_type: The electrical type to get; see power_equipment_electrical_types enumeration
        power_equipment_id: The power equipment to locate; if None, perform on the only power equipment
        electrical_id: The outlet or circuit to locate; if None, perform on the only outlet or circuit

    Returns:
        The outlet or circuit resource
    """

    # Get the power equipment
    power_equipment = get_power_equipment(context, power_equipment_type, power_equipment_id)

    # Check that the power equipment has the desired electrical component type
    if power_equipment_electrical_type.plural not in power_equipment["Info"].dict:
        raise RedfishPowerEquipmentElectricalNotFoundError(
            "{} {} does not contain any {}".format(
                power_equipment_type.plural, power_equipment["Info"].dict["Id"], power_equipment_electrical_type.plural
            )
        )

    power_equipment_electrical_uri_pattern = (
        power_equipment["Info"].dict[power_equipment_electrical_type.plural]["@odata.id"] + "/{}"
    )
    avail_electrical = None

    # If given an identifier, get the outlet or circuit directly
    if electrical_id is not None:
        electrical = context.get(power_equipment_electrical_uri_pattern.format(electrical_id))
    # No identifier given; see if there's exactly one member
    else:
        avail_electrical = get_collection_ids(
            context, power_equipment["Info"].dict[power_equipment_electrical_type.plural]["@odata.id"]
        )
        if len(avail_electrical) == 1:
            electrical = context.get(power_equipment_electrical_uri_pattern.format(avail_electrical[0]))
        else:
            raise RedfishPowerEquipmentElectricalNotFoundError(
                "{} {} contains multiple {}; a target component needs to be specified: {}".format(
                    power_equipment_type.plural,
                    power_equipment["Info"].dict["Id"],
                    power_equipment_electrical_type.plural,
                    ", ".join(avail_electrical),
                )
            )

    # Check the response and return the manager if the response is good
    if electrical.status == 404:
        if avail_electrical is None:
            avail_electrical = get_collection_ids(
                context, power_equipment["Info"].dict[power_equipment_electrical_type.plural]["@odata.id"]
            )
        raise RedfishPowerEquipmentNotFoundError(
            "{} {} does not contain any {} called {}; valid {}: {}".format(
                power_equipment_type.plural,
                power_equipment["Info"].dict["Id"],
                power_equipment_electrical_type.plural,
                electrical_id,
                power_equipment_electrical_type.plural,
                ", ".join(avail_electrical),
            )
        )
    verify_response(electrical)
    return electrical


def print_power_equipment_electrical(electrical):
    """
    Prints the outlet or circuit info

    Args:
        electrical: The outlet or circuit to print
    """

    electrical_line_format = "  {}: {}"
    poly_phase_line_format = "    {:5s}: {}"
    if "#Outlet." in electrical.dict["@odata.type"]:
        print("Outlet {} Info".format(electrical.dict["Id"]))
    else:
        print("{} {} Info".format(electrical.dict["CircuitType"], electrical.dict["Id"]))

    print("")
    # Display the simple properties
    electrical_properties = [
        "Status",
        "CircuitType",
        "CriticalCircuit",
        "ElectricalContext",
        "PhaseWiringType",
        "VoltageType",
        "OutletType",
        "PlugType",
        "NominalVoltage",
        "RatedCurrentAmps",
        "BreakerState",
        "PowerState",
    ]
    for property in electrical_properties:
        if property in electrical.dict:
            prop_val = electrical.dict[property]
            if isinstance(prop_val, list):
                prop_val = ", ".join([i for i in prop_val if i is not None])
            elif property == "PhaseWiringType":
                prop_val = str(phase_wiring_type_strings.get(prop_val, prop_val))
            elif property == "NominalVoltage":
                prop_val = str(voltage_type_strings.get(prop_val, prop_val))
            elif property == "OutletType":
                prop_val = str(receptical_type_strings.get(prop_val, prop_val))
            elif property == "PlugType":
                prop_val = str(plug_type_strings.get(prop_val, prop_val))
            elif property == "Status":
                prop_val = "State: {}, Health: {}".format(prop_val.get("State", "N/A"), prop_val.get("Health", "N/A"))
            else:
                prop_val = str(prop_val)
            print(electrical_line_format.format(property, prop_val))
    # Display the embedded metrics
    electrical_properties = [
        ("Voltage", "Voltage", "V"),
        ("CurrentAmps", "Current", "A"),
        ("PowerWatts", "Power", "W"),
        ("EnergykWh", "Energy", "kWh"),
        ("FrequencyHz", "Frequency", "Hz"),
        ("PowerLoadPercent", "PowerLoad", "%"),
        ("UnbalancedVoltagePercent", "UnbalancedVoltage", "%"),
        ("UnbalancedCurrentPercent", "UnbalancedCurrent", "%"),
    ]
    for property in electrical_properties:
        if property[0] in electrical.dict:
            if electrical.dict[property[0]]["Reading"] is None:
                prop_val = "Unavailable"
            else:
                prop_val = str(electrical.dict[property[0]]["Reading"]) + " " + property[2]
            print(electrical_line_format.format(property[1], prop_val))

        poly_phase_name = "PolyPhase" + property[0]
        if poly_phase_name in electrical.dict:
            if property[0] not in electrical.dict:
                print(electrical_line_format.format(property[1], ""))
            for poly_phase in electrical.dict[poly_phase_name]:
                if electrical.dict[poly_phase_name][poly_phase]["Reading"] is None:
                    prop_val = "Unavailable"
                else:
                    prop_val = (
                        "{:7.2f}".format(electrical.dict["PolyPhase" + property[0]][poly_phase]["Reading"])
                        + " "
                        + property[2]
                    )
                print(poly_phase_line_format.format(line_measurement_strings.get(poly_phase, poly_phase), prop_val))
    print("")


def print_power_equipment_electrical_summary(power_equipment, electrical_type, print_heading):
    """
    Prints the circuit/outlet summary for an instance of power equipment

    Args:
        power_equipment: The power equipment info to print
        electrical_type: The type of electrical info to print
        print_heading: Indicates if the tabling heading should be printed
    """

    if not power_equipment[electrical_type.plural]:
        if print_heading:
            print(
                "{} {} does not contain any {}".format(
                    power_equipment["Info"].dict["EquipmentType"],
                    power_equipment["Info"].dict["Id"],
                    electrical_type.plural,
                )
            )
        return

    electrical_line_format = "  {:16s} | {:18s} | {:20s} | {:12s} | {:12s} | {:12s} | {:12s}"
    if print_heading:
        print(
            "{} {} {}".format(
                power_equipment["Info"].dict["EquipmentType"],
                power_equipment["Info"].dict["Id"],
                electrical_type.plural,
            )
        )
        print("")
    print(
        electrical_line_format.format(
            electrical_type.value + " Id",
            "Nominal Voltage",
            "Phase Wiring",
            "State",
            "Voltage (V)",
            "Current (A)",
            "Power (W)",
        )
    )
    for item in power_equipment[electrical_type.plural]:
        if "PolyPhaseVoltage" in item.dict:
            voltage = "Polyphase"
        else:
            voltage = str(item.dict.get("Voltage", {}).get("Reading", "---"))
        if "PolyPhaseCurrentAmps" in item.dict:
            current = "Polyphase"
        else:
            current = str(item.dict.get("CurrentAmps", {}).get("Reading", "---"))
        voltage_type = item.dict.get("NominalVoltage", "---")
        voltage_type = voltage_type_strings.get(voltage_type, voltage_type)
        phase_wiring_type = item.dict.get("PhaseWiringType", "---")
        phase_wiring_type = phase_wiring_type_strings.get(phase_wiring_type, phase_wiring_type)
        state = str(item.dict.get("Status", {}).get("State", "---"))
        power = str(item.dict.get("PowerWatts", {}).get("Reading", "---"))
        print(
            electrical_line_format.format(
                item.dict["Id"], voltage_type, phase_wiring_type, state, voltage, current, power
            )
        )
    print("")



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Boot Override

File : rf_boot_override.py

Brief : This script uses the redfish_utilities module to manage a one time boot
        override of a system
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to perform a one time boot override of a system")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument("--system", "-s", type=str, help="The ID of the system to set")
argget.add_argument("--info", "-info", action="store_true", help="Indicates if boot information should be reported")
argget.add_argument(
    "--target",
    "-t",
    type=str,
    help="The target boot device; if this argument is omitted the tool will display the current boot settings",
)
argget.add_argument(
    "--uefi",
    "-uefi",
    type=str,
    help="If target is 'UefiTarget', the UEFI Device Path of the device to boot.  If target is 'UefiBootNext', the UEFI Boot Option string of the device to boot.",
)
argget.add_argument("--mode", "-m", type=str, help="The requested boot mode ('UEFI' or 'Legacy')")
argget.add_argument(
    "--reset", "-reset", action="store_true", help="Signifies that the system is reset after the boot override is set"
)
argget.add_argument(
    "--workaround",
    "-workaround",
    action="store_true",
    help="Indicates if workarounds should be attempted for non-conformant services",
    default=False,
)
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
args = argget.parse_args()

# Verify the combination of arguments is correct
if args.target is None:
    args.info = True
    if args.uefi or args.mode or args.reset:
        argget.error("Cannot use '--uefi', '--mode', or '--reset' without '--target'")

if args.workaround:
    redfish_utilities.config.__workarounds__ = True

if args.debug:
    log_file = "rf_boot_override-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_boot_override Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    if args.info:
        boot = redfish_utilities.get_system_boot(redfish_obj, args.system)
        redfish_utilities.print_system_boot(boot)
    else:
        # Build and send the boot request based on the arguments given
        uefi_target = None
        boot_next = None
        boot_enable = "Once"
        if args.target == "UefiTarget":
            uefi_target = args.uefi
        if args.target == "UefiBootNext":
            boot_next = args.uefi
        if args.target == "None":
            print("Disabling one time boot...")
            boot_enable = "Disabled"
        else:
            print("Setting a one time boot for {}...".format(args.target))
        redfish_utilities.set_system_boot(
            redfish_obj, args.system, args.target, boot_enable, args.mode, uefi_target, boot_next
        )

        # Reset the system if requested
        if args.reset:
            print("Resetting the system...")
            response = redfish_utilities.system_reset(redfish_obj, args.system)
            response = redfish_utilities.poll_task_monitor(redfish_obj, response)
            redfish_utilities.verify_response(response)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Configuration Settings

File : config.py

Brief : This file contains configuration settings for the module; useful for
        debugging issues encountered on live systems
"""

# Leverage known workarounds for non-conformant services, such as bypassing
# unexpected 4XX responses, missing properties, and malformed URIs
__workarounds__ = False



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Accounts Module

File : accounts.py

Brief : This file contains the definitions and functionalities for managing
        accounts on a Redfish Service
"""

from .messages import verify_response


class RedfishAccountCollectionNotFoundError(Exception):
    """
    Raised when the Account Service or Account Collection cannot be found
    """

    pass


class RedfishAccountNotFoundError(Exception):
    """
    Raised when the Account Service or Account Collection cannot be found
    """

    pass


class RedfishAccountNotAddedError(Exception):
    """
    Raised when the requested account was not added
    """

    pass


def get_users(context):
    """
    Collects user information from a Redfish service

    Args:
        context: The Redfish client object with an open session

    Returns:
        A list containing all users
    """

    user_list = []

    # Go through each Account in the Account Collection
    account_col = context.get(get_account_collection(context), None)
    for account_member in account_col.dict["Members"]:
        account = context.get(account_member["@odata.id"], None)
        account_info = {
            "UserName": account.dict["UserName"],
            "RoleId": account.dict["RoleId"],
            "Locked": account.dict.get("Locked", False),
            "Enabled": account.dict.get("Enabled", True),
        }

        # Some implementations always expose "slots" for users; ignore empty slots
        if account_info["UserName"] == "" and account_info["Enabled"] is False:
            continue

        user_list.append(account_info)

    return user_list


def print_users(user_list):
    """
    Prints the user list into a table

    Args:
        user_list: The user list to print
    """

    user_line_format = "  {:20s} | {:20s} | {:10s} | {:10s}"
    print("")
    print(user_line_format.format("Name", "Role", "Locked", "Enabled"))
    for user in user_list:
        print(user_line_format.format(user["UserName"], user["RoleId"], str(user["Locked"]), str(user["Enabled"])))
    print("")


def add_user(context, user_name, password, role):
    """
    Adds a new user account

    Args:
        context: The Redfish client object with an open session
        user_name: The name of the user to add
        password: The password for the new user
        role: The role for the new user

    Returns:
        The response of the POST
    """

    # Create the new user
    payload = {"UserName": user_name, "Password": password, "RoleId": role}
    account_col_uri = get_account_collection(context)
    response = context.post(account_col_uri, body=payload)
    if response.status == 405:
        # Some implementations allocate slots for users and don't allow adding in the proper sense
        # Find an empty slot to use
        account_added = False
        account_col = context.get(account_col_uri)
        for account_member in account_col.dict["Members"]:
            account = context.get(account_member["@odata.id"])
            if account.dict["UserName"] == "" and not account.dict.get("Enabled", True):
                # Empty slot found; PATCH it
                response = context.patch(
                    account_member["@odata.id"], body=payload, headers={"If-Match": account.getheader("ETag")}
                )
                if response.status < 400:
                    # These implementations also might restrict which slots to modify...
                    account_added = True
                    break
        if not account_added:
            raise RedfishAccountNotAddedError(
                "Failed to add user '{}'; user may already exist or there is not enough space for the new user".format(
                    user_name
                )
            )
    verify_response(response)
    return response


def delete_user(context, user_name):
    """
    Deletes an existing user account

    Args:
        context: The Redfish client object with an open session
        user_name: The name of the user to delete

    Returns:
        The response of the DELETE
    """

    # Find the user to delete
    user_uri, user_info = get_user(context, user_name)

    # Delete the user
    response = context.delete(user_uri)
    if response.status == 405:
        # Some implementations keep slots around and don't allow for deleting in the proper sense
        # Some also do not allow for both Enabled and UserName to be modified simultaneously
        modify_user(context, user_name, new_enabled=False)
        return modify_user(context, user_name, new_name="")
    verify_response(response)
    return response


def modify_user(
    context,
    user_name,
    new_name=None,
    new_password=None,
    new_role=None,
    new_locked=None,
    new_enabled=None,
    user_uri=None,
):
    """
    Modifies an existing user account

    Args:
        context: The Redfish client object with an open session
        user_name: The name of the user to modify
        new_name: The new name of the user
        new_password: The new password of the user
        new_role: The new role of the user
        new_locked: The new locked flag of the user
        new_enabled: The new enabled flag of the user
        user_uri: The URI of the user to modify

    Returns:
        The response of the PATCH
    """

    # Get the current user info
    user_uri, user_info = get_user(context, user_name, user_uri=user_uri)

    # Build the payload for the new user
    new_info = {}
    if new_name is not None:
        new_info["UserName"] = new_name
    if new_password is not None:
        new_info["Password"] = new_password
    if new_role is not None:
        new_info["RoleId"] = new_role
    if new_locked is not None:
        new_info["Locked"] = new_locked
    if new_enabled is not None:
        new_info["Enabled"] = new_enabled

    # Update the user
    response = context.patch(user_uri, body=new_info, headers={"If-Match": user_info.getheader("ETag")})
    verify_response(response)
    return response


def get_account_collection(context):
    """
    Finds the account collection for the Redfish service

    Args:
        context: The Redfish client object with an open session

    Returns:
        The URI for the account collection
    """

    # Get the Service Root to find the Account Service
    service_root = context.get("/redfish/v1")
    if "AccountService" not in service_root.dict:
        # No Account Service
        raise RedfishAccountCollectionNotFoundError("Service does not contain an Account Service")

    # Get the Account Service to find the Account Collection
    account_service = context.get(service_root.dict["AccountService"]["@odata.id"])
    if "Accounts" not in account_service.dict:
        # No Account Collection
        raise RedfishAccountCollectionNotFoundError("Service does not contain an Account Collection")

    return account_service.dict["Accounts"]["@odata.id"]


def get_user(context, user_name, user_uri=None):
    """
    Finds a user within the Redfish service

    Args:
        context: The Redfish client object with an open session
        user_name: The name of the user to find
        user_uri: The URI of the user to get

    Returns:
        The URI for the user account
        The contents of the user account resource
    """

    avail_users = []

    if user_uri is not None:
        account = context.get(user_uri)
        if account.dict["UserName"] == user_name:
            return user_uri, account

    account_col = context.get(get_account_collection(context))

    for account_member in account_col.dict["Members"]:
        account = context.get(account_member["@odata.id"])

        # Some implementations always expose "slots" for users; ignore empty slots
        if account.dict["UserName"] == "" and not account.dict.get("Enabled", True):
            continue

        avail_users.append(account.dict["UserName"])

        # Check if the name matches
        if account.dict["UserName"] == user_name:
            return account_member["@odata.id"], account

    # No matches found
    raise RedfishAccountCollectionNotFoundError(
        "User '{}' is not found; valid users: {}".format(user_name, ", ".join(avail_users))
    )



#! /usr/bin/python
# Copyright Notice:
# Copyright 2019-2024 DMTF. All rights reserved.
# License: BSD 3-Clause License. For full text see link: https://github.com/DMTF/Redfish-Tacklebox/blob/main/LICENSE.md

"""
Redfish Licenses

File : rf_licenses.py

Brief : This script uses the redfish_utilities module to manage licenses
"""

import argparse
import datetime
import logging
import redfish
import redfish_utilities
import traceback
import sys
from redfish.messages import RedfishPasswordChangeRequiredError

# Get the input arguments
argget = argparse.ArgumentParser(description="A tool to manage licenses on a Redfish service")
argget.add_argument("--user", "-u", type=str, required=True, help="The user name for authentication")
argget.add_argument("--password", "-p", type=str, required=True, help="The password for authentication")
argget.add_argument("--rhost", "-r", type=str, required=True, help="The address of the Redfish service (with scheme)")
argget.add_argument("--debug", action="store_true", help="Creates debug file showing HTTP traces and exceptions")
subparsers = argget.add_subparsers(dest="command")
info_argget = subparsers.add_parser("info", help="Displays information about the licenses installed on the service")
info_argget.add_argument(
    "--details", "-details", action="store_true", help="Indicates if the full details of each license should be shown"
)
install_argget = subparsers.add_parser("install", help="Installs a new license")
install_argget.add_argument(
    "--license", "-l", type=str, required=True, help="The filepath or URI to the license to install"
)
delete_argget = subparsers.add_parser("delete", help="Deletes a license")
delete_argget.add_argument("--license", "-l", type=str, required=True, help="The identifier of the license to delete")
args = argget.parse_args()

if args.debug:
    log_file = "rf_licenses-{}.log".format(datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S"))
    log_format = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    logger = redfish.redfish_logger(log_file, log_format, logging.DEBUG)
    logger.info("rf_licenses Trace")

# Set up the Redfish object
redfish_obj = None
try:
    redfish_obj = redfish.redfish_client(
        base_url=args.rhost, username=args.user, password=args.password, timeout=15, max_retry=3
    )
    redfish_obj.login(auth="session")
except RedfishPasswordChangeRequiredError:
    redfish_utilities.print_password_change_required_and_logout(redfish_obj, args)
    sys.exit(1)
except Exception:
    raise

exit_code = 0
try:
    if args.command == "install":
        print("Installing license '{}'...".format(args.license))
        response = redfish_utilities.install_license(redfish_obj, args.license)
        response = redfish_utilities.poll_task_monitor(redfish_obj, response)
        redfish_utilities.verify_response(response)
    elif args.command == "delete":
        print("Deleting license '{}'...".format(args.license))
        response = redfish_utilities.delete_license(redfish_obj, args.license)
        response = redfish_utilities.poll_task_monitor(redfish_obj, response)
        redfish_utilities.verify_response(response)
    else:
        licenses = redfish_utilities.get_licenses(redfish_obj)
        if args.command == "info":
            redfish_utilities.print_licenses(licenses, details=args.details)
        else:
            redfish_utilities.print_licenses(licenses)
except Exception as e:
    if args.debug:
        logger.error("Caught exception:\n\n{}\n".format(traceback.format_exc()))
    exit_code = 1
    print(e)
finally:
    # Log out
    redfish_utilities.logout(redfish_obj)
sys.exit(exit_code)
